/*  This file is part of the KDE project
    Copyright (C) 2000 Simon Hausmann <hausmann@kde.org>
    Copyright (C) 2003 Luciano Montanaro <mikelima@cirulla.net>
    Copyright (C) 2002 Paul Chitescu <Paul.Chitescu@IMC-Group.org>
    Copyright (C) 2006 basysKom GmbH <info@basyskom.de>
    Copyright (C) 2006 ROAD GmbH <info@road-gmbh.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/


#include "kfiledialog.h"

#include <qlayout.h>
#include <qconfig.h>
#include <qlabel.h>
#include <qtimer.h>

QString KFileDialog::getOpenFileName( const QString &dir,
                                      const QString &filter,
                                      QWidget *parent, const QString &caption )
{
#if defined(_QT_QPE_)
    QPEFileSelectorWrapper *qpefd = new QPEFileSelectorWrapper( parent );
    qpefd->setCaption( caption );
    qpefd->showMaximized();

    QString result;

    if ( qpefd->exec() == QDialog::Accepted )
        result = qpefd->selectedPath(); 

    delete qpefd;

    return result;
#else
#if defined(QT_NO_FILEDIALOG)
    return QString::null; // sorry bud :)
#else
    return QFileDialog::getOpenFileName( dir, filter, parent, 0, caption );
#endif //QT_NO_FILEDIALOG
#endif
}


QString KFileDialog::getSaveFileName( const QString &dir,
                                      const QString &filter,
                                      QWidget *parent, const QString &caption )
{
#if defined(_QT_QPE_)
    QPEFileSaverWrapper *qpefd = new QPEFileSaverWrapper( dir, parent );
    qpefd->setCaption( caption );
    qpefd->showMaximized();

    QString result;

    if ( qpefd->exec() == QDialog::Accepted )
    {
        result = "file:" + qpefd->selectedPath() + "/" + qpefd->selectedFilename(); 
    }

    delete qpefd;

    return result;
#else
#if defined(QT_NO_FILEDIALOG)
    return QString::null; // sorry bud :)
#else
    QFileDialog *qpefd = new QFileDialog( dir, filter, parent, 0, true );
    qpefd->setCaption( caption );
    qpefd->showMaximized();

    QString result;

    if ( qpefd->exec() == QDialog::Accepted )
        result = qpefd->selectedFile(); 

    delete qpefd;
    return result;
//    return QFileDialog::getSaveFileName( dir, filter, parent, 0, caption );
#endif // QT_NO_FILEDIALOG
#endif
}

#if defined(_QT_QPE_) || defined(Q_WS_WIN32)

QPEFileSelectorWrapper::QPEFileSelectorWrapper( QWidget *parent, const char *name )
    : QDialog( parent, name, true )
{
#if defined(_QT_QPE_)
    QVBoxLayout *layout = new QVBoxLayout( this );

    FileSelector *fs = new FileSelector( QString::null, this, "fs" );
    layout->addWidget( fs );

    fs->setNewVisible( false );

    connect( fs, SIGNAL( fileSelected( const DocLnk & ) ),
             this, SLOT( fileSelected( const DocLnk & ) ) );
    connect( fs, SIGNAL( closeMe() ),
             this, SLOT( accept() ) );
#endif
}

QPEFileSelectorWrapper::~QPEFileSelectorWrapper()
{
}

void QPEFileSelectorWrapper::fileSelected( const DocLnk &doc )
{
    m_path = doc.file();
}

QPEFileSaverWrapper::QPEFileSaverWrapper( const QString &dir, QWidget *parent, const char *name )
    : QDialog( parent, name, true )
{
#if defined(_QT_QPE_)
    QVBoxLayout *layout = new QVBoxLayout( this );
    layout->setMargin(4);
    layout->setSpacing(4);

    QHBoxLayout* layoutpath = new QHBoxLayout(layout);

    QLabel* label = new QLabel(this);
    label->setText(tr("Storage"));
    layoutpath->addWidget(label);

    m_storageCombo = new QComboBox(this);
    m_storageCombo->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred));
    layoutpath->addWidget(m_storageCombo);

    QHBoxLayout* layoutfile = new QHBoxLayout(layout);
   
    label = new QLabel(this);
    label->setText(tr("Filename"));
    layoutfile->addWidget(label);

    m_fileEdit = new QLineEdit(this);
    m_fileEdit->setText(dir);
    layoutfile->addWidget(m_fileEdit);

    layout->addStretch();

    storage = new StorageInfo( this );
    connect( storage, SIGNAL( disksChanged() ), SLOT( updateMemory() ) );
    QTimer::singleShot(0, this, SLOT(updateMemory()));
#endif
}

QPEFileSaverWrapper::~QPEFileSaverWrapper()
{
}


void QPEFileSaverWrapper::updateMemory()
{
    memory.clear();
    nameMap.clear();

    QDir docDir( QDir::homeDirPath() );
    if ( docDir.exists() )
    {
        QString name = StorageInfo::tr("Internal Storage");
        memory.append( name );
        nameMap[name] = docDir.path();
    }

    // insert all removable disks and the home partion 
    const QList<FileSystem> &fs = storage->fileSystems();
    QListIterator<FileSystem> it ( fs );
    for ( ; it.current(); ++it ){
        if ( (*it)->isRemovable() ) {
            memory.append( (*it)->name() );
            nameMap[(*it)->name()] = (*it)->path();
        } 
    }

    m_storageCombo->clear();
    m_storageCombo->insertStringList(memory);
}


QString QPEFileSaverWrapper::selectedPath() const 
{ 
    kdDebug() << "Selected Path: " << nameMap[m_storageCombo->currentText()] << endl;
    return nameMap[m_storageCombo->currentText()]; 
}

QString QPEFileSaverWrapper::selectedFilename() const 
{ 
    kdDebug() << "Selected Filename: " << m_fileEdit->text() << endl;
    return m_fileEdit->text(); 
}


#include "kfiledialog.moc"
#endif
