/*  This file is part of the KDE project
    Copyright (C) 2000 Simon Hausmann <hausmann@kde.org>
    Copyright (C) 2003 Luciano Montanaro <mikelima@cirulla.net>
    Copyright (C) 2001 Dirk Mueller <mueller@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "khtml_ext.h"

#include "khtmlview.h"
#include "khtml_pagecache.h"

#include <qapplication.h>
#include <qpopupmenu.h>

#include <kdebug.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <kio/job.h>
#include <ktempfile.h>
#include <ksavefile.h>
#include <kmessagebox.h>

#include <dom/dom_element.h>
#include <misc/htmltags.h>

KHTMLPartBrowserExtension::KHTMLPartBrowserExtension( KHTMLPart *parent, const char *name )
: KParts::BrowserExtension( parent, name )
{
  m_part = parent;
  m_part->setDNDEnabled( false ); // disable dnd in here, because then it is
                                  // also disabled for all frames
}

int KHTMLPartBrowserExtension::xOffset()
{
    return m_part->view()->contentsX();
}

int KHTMLPartBrowserExtension::yOffset()
{
  return m_part->view()->contentsY();
}

void KHTMLPartBrowserExtension::saveState( QDataStream &stream )
{
  kdDebug( 6050 ) << "saveState!" << endl;
  m_part->saveState( stream );
}

void KHTMLPartBrowserExtension::restoreState( QDataStream &stream )
{
  kdDebug( 6050 ) << "restoreState!" << endl;
  m_part->restoreState( stream );
}

void KHTMLPartBrowserExtension::reparseConfiguration()
{
  m_part->reparseConfiguration();
}

void KHTMLPartBrowserExtension::print()
{
  m_part->view()->print();
}

KHTMLPartBrowserHostExtension::KHTMLPartBrowserHostExtension( KHTMLPart *part )
: KParts::BrowserHostExtension( part )
{
  m_part = part;
}

KHTMLPartBrowserHostExtension::~KHTMLPartBrowserHostExtension()
{
}

QStringList KHTMLPartBrowserHostExtension::frameNames() const
{
  return m_part->frameNames();
}

const QPtrList<KParts::ReadOnlyPart> KHTMLPartBrowserHostExtension::frames() const
{
  return m_part->frames();
}

bool KHTMLPartBrowserHostExtension::openURLInFrame( const KURL &url, const KParts::URLArgs &urlArgs )
{
  return m_part->openURLInFrame( url, urlArgs );
}

//FW, ML, XXX Check if we need this after all...
int KHTMLZoomFactorAction::plug( QWidget * /*w*/, int /*index*/ )
{
    return -1;
}

void KHTMLPopupGUIClient::saveURL( QWidget *parent, const QString &caption,
                                   const KURL &url,
                                   const QMap<QString, QString> &metadata,
                                   const QString &filter, long cacheId,
                                   const QString & suggestedFilename, const QString& mimeType )
{
  QString name = QString::fromLatin1( "index.html" );
  if ( !suggestedFilename.isEmpty() )
    name = suggestedFilename;
  else if ( !url.fileName().isEmpty() )
    name = url.fileName();

  KURL destURL;
  DocLnk f;
  int query;
  do {
    query = KMessageBox::Yes;
      destURL = KFileDialog::getSaveURL( name, filter, parent, caption );
      if (destURL.isEmpty())
         return;
#ifdef _QT_QPE_
      QStringList types = QStringList::split('/', mimeType);
      QString major = types[0];
      QString minor = types[1];

      // creating needed directoriess
      QFileInfo fi = destURL.path();
      QDir dir = fi.dir();
      if (!dir.exists("Documents")) dir.mkdir("Documents");
      dir = dir.absPath() + "/Documents";
      if (!dir.exists(major)) dir.mkdir(major);
      dir = dir.absPath() + "/" + major;
      if (!dir.exists(minor)) dir.mkdir(minor);
      dir = dir.absPath() + "/" + minor + "/";
      destURL = dir.absPath() + "/" + destURL.fileName();

      f.setLocation(dir.absPath());
      f.setType(mimeType);
      f.setName(destURL.fileName());
      f.setFile(destURL.fileName());
      f.setLinkFile(destURL.path()+".desktop");
#endif
      kdDebug() << "***** about to save file to " << destURL << endl;
      if( destURL.isLocalFile() )
      {
        QFileInfo info( destURL.path() );
        if( info.exists() ) {
          // TODO: use KIO::RenameDlg (shows more information)
          query = KMessageBox::warningYesNoCancel( parent, i18n( "A file named \"%1\" already exists. " "Are you sure you want to overwrite it?" ).arg( info.fileName() ), i18n( "Overwrite File?" ), i18n( "Overwrite" ) );
        }
       }
   } while ( query == KMessageBox::No );
  if (query == KMessageBox::Cancel)
    return;

  if ( destURL.isValid() )
    saveURL(url, destURL, metadata, cacheId, f);
}

void KHTMLPopupGUIClient::saveURL( const KURL &url, const KURL &destURL,
                                   const QMap<QString, QString> &metadata,
                                   long cacheId, DocLnk f )
{
    if ( destURL.isValid() )
    {
        bool saved = false;
        if (KHTMLPageCache::self()->isComplete(cacheId))
        {
            // QFileDialog gives us only local files as destination => copy
            KSaveFile destFile(destURL.path());
            if (destFile.status() == 0)
            {
                kdDebug() << "Download was in cache" << endl;
                KHTMLPageCache::self()->saveData(cacheId, destFile.dataStream());
                saved = true;
            }
        }
        if(!saved)
        {
            kdDebug() << "Saving file from " << url << " to " << destURL << endl;
            KIO::TransferJob *job = KIO::file_copy( url, destURL, f, -1, true /*overwrite*/, false, true );
            job->setMetaData(metadata);
            job->addMetaData("MaxCacheSize", "0"); // Don't store in http cache.
            job->addMetaData("cache", "cache"); // Use entry from cache if available. 
//            job->setAutoErrorHandlingEnabled( true );  
            QObject::connect(kapp, SIGNAL(aboutToQuit()), job, SLOT( kill() ));

            ProgressDlg* defaultProgress = new ProgressDlg(kapp->mainWidget());
            defaultProgress->slotTransferring( job, url );
            defaultProgress->setOnlyClean( true );
            defaultProgress->setJob(job);

            QObject::connect( job, SIGNAL( percent( KIO::Job*, unsigned long ) ),
                 defaultProgress, SLOT( slotPercent( KIO::Job*, unsigned long ) ) );
            QObject::connect( job, SIGNAL( infoMessage( KIO::Job*, const QString & ) ),
                 defaultProgress, SLOT( slotInfoMessage( KIO::Job*, const QString & ) ) );
            QObject::connect( job, SIGNAL( totalSize( KIO::Job*, KIO::filesize_t ) ),
                 defaultProgress, SLOT( slotTotalSize( KIO::Job*, KIO::filesize_t ) ) );
            QObject::connect( job, SIGNAL( processedSize( KIO::Job*, KIO::filesize_t ) ),
                 defaultProgress, SLOT( slotProcessedSize( KIO::Job*, KIO::filesize_t ) ) );
            QObject::connect( job, SIGNAL( speed( KIO::Job*, unsigned long ) ),
                 defaultProgress, SLOT( slotSpeed( KIO::Job*, unsigned long ) ) );

            defaultProgress->show();

        } //end if(!saved)
    }
}


#include "khtml_ext.moc"


