/*  This file is part of the KDE project
    Copyright (C) 2000 Simon Hausmann <hausmann@kde.org>
    Copyright (C) 1999 Torben Weis <weis@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    As a special exception this program may be linked with Qt non-commercial 
    edition, the resulting executable be distributed, without including the 
    source code for the Qt non-commercial edition in the source distribution.

*/

#include "kprotocolmanager.h"
#include <kglobal.h>
#include <kconfig.h>
#include <ioslave_defaults.h>
#include <kdeversion.h>
#include <qglobal.h>

#include <stdlib.h>
#include <sys/utsname.h>


#ifdef _QT_QPE_
#include <qtopia/config.h>
#include <qtopia/global.h>
#endif

// CACHE SETTINGS
#define DEFAULT_MAX_CACHE_SIZE          5120          //  5 MB
#define DEFAULT_MAX_CACHE_AGE           60*60*24*14   // 14 DAYS
#define DEFAULT_EXPIRE_TIME             60*30         // 1/2 hour

// MAXIMUM VALUE ALLOWED WHEN CONFIGURING
// REMOTE AND PROXY SERVERS CONNECTION AND
// RESPONSE TIMEOUTS.
#define MAX_RESPONSE_TIMEOUT            360           //  6 MIN
#define MAX_CONNECT_TIMEOUT             360           //  6 MIN
#define MAX_PROXY_CONNECT_TIMEOUT       120           //  2 MIN

#ifdef Q_WS_QWS
#define DEFAULT_USERAGENT_STRING \
   QString::fromLatin1("Mozilla/5.0 (compatible; Konqueror/%1) KHTML/%2 (like Gecko); QtEmbedded/%3x%4") \
   .arg( KDE_VERSION_STRING ).arg( KDE_VERSION_STRING ).arg( qApp->desktop()->width() ).arg( qApp->desktop()->height() )
#else
#define DEFAULT_USERAGENT_STRING \
   QString::fromLatin1("Mozilla/5.0 (compatible; Konqueror/%1; X11) KHTML/%2 (like Gecko)").arg( KDE_VERSION_STRING ).arg( KDE_VERSION_STRING )
#endif


//from KDE sources:
//#define CFG_DEFAULT_UAGENT(X) \
//QString("Mozilla/5.0 (compatible; Konqueror/%1.%2%3) KHTML/%4.%5.%6 (like Gecko)") \
//        .arg(KDE_VERSION_MAJOR).arg(KDE_VERSION_MINOR).arg(X).arg(KDE_VERSION_MAJOR).arg(KDE_VERSION_MINOR).arg(KDE_VERSION_RELEASE)

#define QFL(x) QString::fromLatin1(x)

QString KProtocolManager::parseUADescription(const QString& uastring)
{
  QString tmp(uastring);
  kdDebug() << "*************** PARSE USER AGENT STRING (" << uastring << ")" << endl;

      struct utsname utsn;
      uname( &utsn );

      tmp.replace( QFL("appSysName"), QString(utsn.sysname) );
      tmp.replace( QFL("appSysRelease"), QString(utsn.release) );
      tmp.replace( QFL("appMachineType"), QString(utsn.machine) );

      QStringList languageList = KGlobal::locale()->languageList();
      if ( languageList.count() )
      {
        QStringList::Iterator it = languageList.find( QString::fromLatin1("C") );
        if( it != languageList.end() )
        {
          if( languageList.contains( QString::fromLatin1("en") ) > 0 )
            languageList.remove( it );
          else
            (*it) = QString::fromLatin1("en");
        }
      }

      tmp.replace( QFL("appLanguage"), QString("%1").arg(languageList.join(", ")) );
      tmp.replace( QFL("appPlatform"), QFL("X11") );

   return tmp;
}



QString KProtocolManager::userAgentForHost( const QString &/*host*/ )
{
    QString agent = QString::fromLatin1( getenv( "HTTP_USER_AGENT" ) );
    if ( !agent.isEmpty() )
	return agent;

    KConfig *config = KGlobal::config();
    KConfigGroupSaver saver( config, QString::fromLatin1( "General" ) );
    agent = config->readEntry( "UserAgentString" );
    if ( !agent.isEmpty() )
	return parseUADescription(agent);

    return DEFAULT_USERAGENT_STRING;
}

QString KProtocolManager::defaultUserAgent( const QString & ) 
{ 
    return userAgentForHost();
}


bool KProtocolManager::useProxy()
{
    return !httpProxy().isEmpty();
}

QString KProtocolManager::httpProxy()
{
    KConfig *config = KGlobal::config();
    KConfigGroupSaver saver( config, QString::fromLatin1( "Network Settings" ) );

#ifdef _QT_QPE_
// use Qtopia settings (different for each network setup)
    Config proxy( Global::applicationFileName("Network", "modules") + "/Proxies.conf", Config::File );
    proxy.setGroup("Properties");

// this is how Qtopia reads the stuff:
/*    proxy.setGroup("Info");
    proxy.writeEntry("UserSelectable",0);
    proxy.setGroup("Properties");
    proxy.writeEntry("type", cfg.readEntry("type") );
    proxy.writeEntry("autoconfig", cfg.readEntry("autoconfig") );
    proxy.writeEntry("httphost", cfg.readEntry("httphost") );
    proxy.writeEntry("httpport", cfg.readEntry("httpport") );
    proxy.writeEntry("ftphost", cfg.readEntry("ftphost") );
    proxy.writeEntry("ftpport", cfg.readEntry("ftpport") );
    proxy.writeEntry("noproxies", cfg.readEntry("noproxies") );
*/
    switch (proxy.readNumEntry("type"))
    {
       case 1: // auto config
          config->writeEntry( "Proxy Config Script", proxy.readEntry("autoconfig"));
          config->writeEntry( "ProxyType", 2);
          break;

       case 2: // manual
          proxy.setGroup("Properties");
          //if ( protocol.left( 4 ) == "http" )
          config->writeEntry( "httpProxy", QString("http://") + proxy.readEntry("httphost") + ":" + proxy.readEntry("httpport") );
          //if ( protocol.left( 3 ) == "ftp" )
          config->writeEntry( "ftpProxy", QString("ftp://") + proxy.readEntry("ftphost") + ":" + proxy.readEntry("ftpport") );
          config->writeEntry( "NoProxyFor", proxy.readEntry("noproxies"));
          config->writeEntry( "ProxyType", 1);
          break;

       case 0: // no proxy
       default:
          config->writeEntry( "ProxyType", 0);
          break;
      }
#endif

    // get proxystring from settings
    QString proxystring;
    QString res;
    switch (config->readNumEntry( "ProxyType" ))
    {
        case 1: // settings
           proxystring = config->readEntry( "httpProxy" );
           break;
        case 3: // environment variable
           res = QString::fromLatin1( getenv( "HTTP_PROXY" ) );
           if ( res.isEmpty() )
             res = QString::fromLatin1( getenv( "http_proxy" ) );
           if ( !res.isEmpty() )
             proxystring = res;
           break;
        case 2:
        case 0: //no proxy
        default:
          proxystring = QString::null;
    }
    kdDebug() << "using PROXY: " << proxystring << endl;
    return proxystring;
}

void KProtocolManager::setHTTPProxy( const QString &server )
{
    KConfig *config = KGlobal::config();
    KConfigGroupSaver saver( config, QString::fromLatin1( "Network Settings" ) );
    config->writeEntry( "httpProxy", server );
}

QString KProtocolManager::ftpProxy()
{
    KConfig *config = KGlobal::config();
    KConfigGroupSaver saver( config, QString::fromLatin1( "Network Settings" ) );

    // get proxystring from settings
    QString proxystring;
    QString res;
    switch (config->readNumEntry( "ProxyType" ))
    {
        case 1: // settings
           proxystring = config->readEntry( "ftpProxy" );
           break;
        case 3: // environment variable
           res = QString::fromLatin1( getenv( "FTP_PROXY" ) );
           if ( res.isEmpty() )
             res = QString::fromLatin1( getenv( "ftp_proxy" ) );
           if ( !res.isEmpty() )
             proxystring = res;
           break;
        case 2:
        case 0: //no proxy
        default:
          proxystring = QString::null;
    }
    return proxystring;
}

QString KProtocolManager::noProxyFor()
{
    return QString::fromLatin1( getenv( "NO_PROXY_FOR" ) );
}

QString KProtocolManager::proxyFor( const QString &protocol )
{
    if ( protocol.left( 4 ) == "http" )
        return httpProxy();
    else if ( protocol.left( 3 ) == "ftp" )
        return ftpProxy();
    else
        return QString::null;
}

bool KProtocolManager::useCache()
{
    return getenv( "KIO_HTTP_USECACHE" ) != 0;
}

int KProtocolManager::maxCacheAge()
{
    return configValue( "KIO_HTTP_MAXCACHEAGE", DEFAULT_MAX_CACHE_AGE );
}

int KProtocolManager::maxCacheSize()
{
    return configValue( "KIO_HTTP_MAXCACHESIZE", DEFAULT_MAX_CACHE_SIZE );
}

int KProtocolManager::proxyConnectTimeout()
{
    return configValue( "KIO_HTTP_PROXY_CONNECT_TIMEOUT", DEFAULT_PROXY_CONNECT_TIMEOUT );
}

int KProtocolManager::connectTimeout()
{
    return configValue( "KIO_HTTP_CONNECT_TIMEOUT", DEFAULT_CONNECT_TIMEOUT );
}

int KProtocolManager::responseTimeout()
{
    return configValue( "KIO_HTTP_RESPONSE_TIMEOUT", DEFAULT_RESPONSE_TIMEOUT );
}

int KProtocolManager::readTimeout()
{
    return configValue( "KIO_HTTP_READ_TIMEOUT", DEFAULT_READ_TIMEOUT );
}

int KProtocolManager::configValue( const char *envVarName, int defaultVal )
{
    QString valStr = QString::fromLatin1( getenv( envVarName ) );
    bool ok = false;

    int val = valStr.toInt( &ok );

    if ( !valStr.isEmpty() && ok )
        return val;

    return defaultVal;
}

KIO::MetaData KProtocolManager::protocolConfig( const QString &protocol )
{
    KIO::MetaData res;

    // set User Agent
    res.insert("UserAgent", defaultUserAgent() );

    // set Proxy 
    KConfig *config = KGlobal::config();
    QString group = protocol.lower() + " Settings";
    KConfigGroupSaver saver( config, group );
   
    res.insert( "UseProxy", proxyFor( protocol ) );

    res += config->entryMap( group ); 

    // set cookie information
    // arghl, temporary hack (need kio::sessiondata, kind of)
    if ( !res.contains( "Cookies" ) )
        res.insert( "Cookies", "true" );

    return res;
/*
    KIO::MetaData res;
    res.insert( "UseProxy", proxyFor( protocol ) );                 

    // ### hack
    if ( protocol.startsWith( "http" ) )
    {
        res.insert( "UseCache", useCache() ? "true" : "false" );
        res.insert( "MaxCacheAge", QString::number( maxCacheAge() ) );
    }

    return res;
*/
}

QString KProtocolManager::externalProtocolHandler( const QString &protocol )
{
    KConfig *config = KGlobal::config();
    KConfigGroupSaver saver( config, QString::fromLatin1( "External Protocols" ) );

    return config->readEntry( protocol, QString::null );
}

