/* This file is part of the KDE libraries
   Copyright (C) 2000 Matej Koss <koss@miesto.sk>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include <qlayout.h>
#include <qdatetime.h>

#include <kapplication.h>
#include <kdebug.h>
#include <kdialog.h>
#include <kstringhandler.h>
#include <kglobal.h>
#include <klocale.h>
#include <kpushbutton.h>
//#include <kstandarddirs.h>
#include <klineedit.h>

#include "jobclasses.h"
#include "progressdlg.h"

//namespace KIO {

class ProgressDlg::ProgressDlgPrivate
{
public:
  bool keepOpenChecked;
  bool noCaptionYet;
  KPushButton *cancelClose;
  KURL        location;
  QTime       startTime;
};


ProgressDlg::ProgressDlg( QWidget* parent, const char* name, bool modal, WFlags f )
  : ProgressBase( parent, name, modal, f ),
  m_iTotalSize(0),
  m_iProcessedSize(0)
{
    init();
}

void ProgressDlg::init()
{
  d = new ProgressDlgPrivate;

  QVBoxLayout *topLayout = new QVBoxLayout( this, KDialog::marginHint(),
                                            KDialog::spacingHint() );
  topLayout->addStrut( 480 );   // makes dlg at least that wide

  QGridLayout *grid = new QGridLayout( 2, 3 );
  topLayout->addLayout(grid);
  grid->addColSpacing(1, KDialog::spacingHint());
  // filenames or action name
  grid->addWidget(new QLabel(i18n("Download:"), this), 0, 0);

  sourceEdit = new KLineEdit(this);
  sourceEdit->setReadOnly(true);
  grid->addWidget(sourceEdit, 0, 2);

  m_pProgressBar = new QProgressBar(this);
  topLayout->addWidget( m_pProgressBar );
  grid->addRowSpacing(1,10);

  // processed info
  QHBoxLayout *hBox = new QHBoxLayout();
//  topLayout->addLayout(hBox);

//  sizeLabel = new QLabel(this);
//  hBox->addWidget(sizeLabel);

//  resumeLabel = new QLabel(this);
//  hBox->addWidget(resumeLabel);

//  progressLabel = new QLabel( this );
//  progressLabel->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding,
//                                             QSizePolicy::Preferred ) );
//  progressLabel->setAlignment( QLabel::AlignRight );
//  hBox->addWidget( progressLabel );

//  hBox = new QHBoxLayout();
  topLayout->addLayout(hBox);

  speedLabel = new QLabel(this);
  hBox->addWidget(speedLabel, 1);

  QFrame *line = new QFrame( this );
  line->setFrameShape( QFrame::HLine );
  line->setFrameShadow( QFrame::Sunken );
  topLayout->addWidget( line );

  hBox = new QHBoxLayout();
  topLayout->addLayout(hBox);

  hBox->addStretch(1);

  d->cancelClose = new KPushButton( i18n("Abort"), this );
  connect( d->cancelClose, SIGNAL( clicked() ), this, SLOT( slotStop() ) );
  hBox->addWidget( d->cancelClose );

  resize( sizeHint() );
  setMaximumHeight(sizeHint().height());

  d->keepOpenChecked = false;
  d->noCaptionYet = true;
  setCaption(i18n("Download Progress")); // show something better than kio_uiserver
}

ProgressDlg::~ProgressDlg()
{
  delete d;
}

void ProgressDlg::slotTotalSize( KIO::Job*, KIO::filesize_t size )
{
  // size is measured in bytes
  if ( m_iTotalSize == size )
      return;
  m_iTotalSize = size;
  if (d->startTime.isNull())
    d->startTime.start();
}


//static
QString ProgressDlg::makePercentString( unsigned long percent,
                                            KIO::filesize_t totalSize )
{
  if ( totalSize )
      return i18n( "%1 % of %2 " ).arg( percent ).arg( KIO::convertSize( totalSize ) );
  else
      return i18n( "%1 %" ).arg( percent );
}

void ProgressDlg::slotPercent( KIO::Job*, unsigned long percent )
{
  QString caption = makePercentString( percent, m_iTotalSize );
  m_pProgressBar->setProgress( percent );
  switch(mode) {
  case Copy:
    caption.append(i18n(" (Downloading)"));
    break;
  case Done:
    caption.append(i18n(" (Done)"));
    break;
  }

  setCaption( caption );
  d->noCaptionYet = false;
}


void ProgressDlg::slotInfoMessage( KIO::Job*, const QString & msg )
{
  speedLabel->setText( msg );
  speedLabel->setAlignment( speedLabel->alignment() & ~Qt::WordBreak );
}


void ProgressDlg::slotProcessedSize( KIO::Job*, KIO::filesize_t bytes ) {
/*  if ( m_iProcessedSize == bytes )
    return;
  m_iProcessedSize = bytes;

  QString tmp = i18n( "%1 of %2 complete")
                .arg( KIO::convertSize(bytes) )
                .arg( KIO::convertSize(m_iTotalSize));
  sizeLabel->setText( tmp );*/
}


void ProgressDlg::slotSpeed( KIO::Job*, unsigned long speed )
{
  if ( speed == 0 ) {
    speedLabel->setText( i18n( "Stalled") );
  } else {
    speedLabel->setText( i18n( "%1/s ( %2 remaining )").arg( KIO::convertSize( speed ))
        .arg( KIO::convertSeconds( KIO::calculateRemainingSeconds( m_iTotalSize, m_iProcessedSize, speed ))) );
  }
}


void ProgressDlg::slotCopying( KIO::Job*, const KURL& from, const KURL& to )
{
  if ( d->noCaptionYet ) {
    setCaption(i18n("Copy File Progress"));
    d->noCaptionYet = false;
  }
  mode = Copy;
  sourceEdit->setText(from.prettyURL());
}



void ProgressDlg::slotTransferring( KIO::Job*, const KURL& url )
{
  if ( d->noCaptionYet ) {
    setCaption(i18n("Loading Progress"));
    d->noCaptionYet = false;
  }
  sourceEdit->setText(url.prettyURL());
}

void ProgressDlg::slotCanResume( KIO::Job*, KIO::filesize_t resume )
{
/*  if ( resume ) {
    resumeLabel->setText( i18n("Resuming from %1").arg(KIO::number(resume)) );
  } else {
    resumeLabel->setText( i18n("Not resumable") );
  }*/
}

void ProgressDlg::slotClean()
{
  QTimer::singleShot( 1000, this, SLOT( slotDone() ) );
}

void ProgressDlg::slotDone()
{
  accept();
}


//}

#include "progressdlg.moc"
