/*  This file is part of the KDE project
    Copyright (C) 2005-2006 ROAD Gmbh <info@road-gmbh.de>
    Copyright (C) 2005-2006 basysKom Gmbh <info@basyskom.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    As a special exception this program may be linked with Qt non-commercial 
    edition, the resulting executable be distributed, without including the 
    source code for the Qt non-commercial edition in the source distribution.

*/

#if defined(ENABLE_BOOKMARKS)

#include "bookmarkeditorhierarchical.h"
#include <assert.h>
#include "bookmarks.h"

//////// HELPER FUNCTIONS /////////////////////////////////
namespace {

QListViewItem *firstSibling( QListViewItem *item)
{
    if (!item)
        return 0;
    return (item->parent())
               ? item->parent()->firstChild()
               : item->listView()->firstChild();
}

QListViewItem *prevSibling( QListViewItem *item )
{
    QListViewItem *sibling = firstSibling( item );
    if (sibling == item)
        return 0; // the given item has no previous sibling
    while( sibling && sibling->nextSibling() != item )
        sibling = sibling->nextSibling();
    return sibling;
}

template <class ParentClass>
void reinsert( QListViewItem *item, ParentClass *parent )
{
    parent->takeItem( item );
    parent->insertItem( item );
}

void moveToFirst( BookmarkItem *item )
{
    BookmarkItem *firstSib = static_cast<BookmarkItem*>( firstSibling( item ) );
    if (firstSib == item)
       return;

    (item->parent())
        ? reinsert( item, item->parent() )
        : reinsert( item, item->listView() );
    item->listView()->setSelected( item, true );

    XMLElement *e = item->bookmark().xmlElement();
    XMLElement *refE = firstSib->bookmark().xmlElement();
    refE->parent()->insertBefore( e, refE );
}

void moveAfter( BookmarkItem *item, BookmarkItem *moveAfter )
{
    assert( item && moveAfter );
    assert( item->parent() == moveAfter->parent() );

    item->moveItem( moveAfter );

    XMLElement *e = item->bookmark().xmlElement();
    XMLElement *refE = moveAfter->bookmark().xmlElement();
    refE->parent()->insertAfter( e, refE );
}

void moveBefore( BookmarkItem *item, QListViewItem *moveBefore )
{
    assert( item && moveBefore );
    assert( item->parent() == moveBefore->parent() );
    BookmarkItem *moveAfterItem = static_cast<BookmarkItem *>( prevSibling( moveBefore ) );
    if (moveAfterItem == item)
        return;
    (moveAfterItem)
        ? moveAfter( item, moveAfterItem )
        : moveToFirst( item );
}

template <class ParentClass>
void doRefill( const BookmarkFolder& folder, ParentClass *parent )
{
    QListViewItem *last = 0;
    Bookmark bm = folder.first();
    for (; !bm.isNull(); bm = folder.next( bm ) )
    {
        last = new BookmarkItem( parent, last, bm );
        if ( bm.isFolder() )
            doRefill( bm.toFolder(), last );
    }
}


// returns the folder directly below the given item, or null if the next item
// doesn't exist of isn't a folder.
BookmarkItem *folderBelow( QListViewItem *item )
{
    BookmarkItem *retVal = 0;
    if (item)
    {
        BookmarkItem *folder = static_cast<BookmarkItem *>( item->nextSibling() );
        if (folder && folder->bookmark().isFolder())
            retVal = folder;
    }
    return retVal;
}

}; // anonymous namespace
//////// BookmarkEditorHierarchical FUNCTIONS /////////////////////////////////

BookmarkEditorHierarchical::BookmarkEditorHierarchical( XMLElement *bookmarks, QWidget *parent, const char *name )
    : BookmarkEditor( bookmarks, parent, name )
{
    gotoBookmarkButton->show();
    Line1_2->show();
    newFolderButton->show();
    moveBookmarkOutOfFolderButton->show();
    moveBookmarkIntoFolderButton->show();
    view->setRootIsDecorated( true );
    refill();
}

void BookmarkEditorHierarchical::gotoBookmark()
{
    BookmarkItem *item = static_cast<BookmarkItem *>( view->selectedItem() );
    if (item)
    {
        const Bookmark& bm = item->bookmark();
        if (!bm.isFolder())
            emit gotoBookmarkSignal( bm.url() );
    }
    accept();
}

void BookmarkEditorHierarchical::moveBookmarkDown()
{
    // moving down doesn't change levels in the bookmark hierarchy
    BookmarkItem *item = static_cast<BookmarkItem *>( view->selectedItem() );
    moveAfter( item,
               static_cast<BookmarkItem *>( item->nextSibling() ) );
    view->ensureItemVisible( item );
    updateButtons();
}

void BookmarkEditorHierarchical::moveBookmarkUp()
{
    // moving up doesn't change levels in the bookmark hierarchy
    BookmarkItem *item = static_cast<BookmarkItem *>( view->selectedItem() );
    moveBefore( item, prevSibling( item ) );
    view->ensureItemVisible( item );
    updateButtons();
}

void BookmarkEditorHierarchical::moveBookmarkIntoFolder()
{
    // if there is a folder directly beneath this bookmark, move the bookmark
    // into it; otherwise, do nothing.
    BookmarkItem *item = static_cast<BookmarkItem *>( view->selectedItem() );
    BookmarkItem *newParentFolder = folderBelow( item );
    if (newParentFolder)
    {
        setParent( item, newParentFolder );
        view->ensureItemVisible( item );
        updateButtons();
    }
}

void BookmarkEditorHierarchical::moveBookmarkOutOfFolder()
{
    BookmarkItem *item = static_cast<BookmarkItem *>( view->selectedItem() );
    if (!item)
        return;
    QListViewItem *parent = item->parent();
    if (parent)
    {
        setParent( item, static_cast<BookmarkItem *>( parent->parent() ) );
        moveBefore( item, parent ); // otherwise we end up as our grandparent's *first* child
        view->ensureItemVisible( item );
        updateButtons();
    }
}


void BookmarkEditorHierarchical::newFolder()
{
    BookmarkEdit *edit = new BookmarkEdit( this );
    edit->setCaption( i18n( "New Folder" ) );
    edit->location->setEnabled(false);
    edit->location->hide();
    edit->TextLabel2->hide();
    if ( edit->exec() == QDialog::Accepted )
        doNewBookmark( edit, "folder" );
    delete edit;
}

void BookmarkEditorHierarchical::updateButtons()
{
    BookmarkEditor::updateButtons();
    QListViewItem *it = view->selectedItem();

    const bool haveItem = it != 0;
    const bool itIsFirst = haveItem ? firstSibling(it) == it : true;
    const bool itIsLast = haveItem ? !it->nextSibling() : true;
    const bool itIsAboveFolder = folderBelow(it);
    const bool itIsInFolder = haveItem ? it->parent() : false;

    moveUpButton->setEnabled( !itIsFirst );
    moveDownButton->setEnabled( !itIsLast );
    moveBookmarkIntoFolderButton->setEnabled( itIsAboveFolder );
    moveBookmarkOutOfFolderButton->setEnabled( itIsInFolder );
}

void BookmarkEditorHierarchical::refill()
{
    view->clear();

    BookmarkFolder root( m_bookmarks->firstChild() );
    if ( !root.isFolder() )
        return;

    doRefill( root, view );

    updateButtons();
}

void BookmarkEditorHierarchical::setParent( BookmarkItem *item, BookmarkItem *newParent )
{
    // if newParent is NULL, the item will be moved to the top level
    assert( item );
    QListViewItem *oldParent = item->parent();
    if(oldParent == newParent)
        return;
    QListView *listView = item->listView();
    (oldParent)
        ? oldParent->takeItem( item )
        : listView->takeItem( item );
    (newParent)
        ? newParent->insertItem( item )
        : listView->insertItem( item );
    listView->setSelected( item, true );

    XMLElement *e = item->bookmark().xmlElement();
    XMLElement *refE = (newParent)
        ? newParent->bookmark().xmlElement()
        : m_bookmarks->firstChild();
    refE->insertBefore( e, refE->firstChild() );
}
#include "bookmarkeditorhierarchical.moc"

#endif //defined ENABLE_BOOKMARKS

/*
 * vim:et
 */
