/*  This file is part of the KDE project
    Copyright (C) 2005-2006 ROAD Gmbh <info@road-gmbh.de>
    Copyright (C) 2005-2006 basysKom Gmbh <info@basyskom.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "roadtabbar.h"

RoadTabParams::RoadTabParams( const QIconSet &defaultIS, const QIconSet &loadingIS,
                              const QFontMetrics& fm, int mw ) :
    defaultIconSet( defaultIS ),
    loadingIconSet( loadingIS ),
    fontMetrics( fm ),
    maxWidth( mw )
{}

RoadTabBar::RoadTabBar( const QIconSet &defaultIS, const QIconSet &loadingIS,
                        QWidget *parent, const char *name ) :
    QTabBar( parent, name ),
    m_contextMenuTabIdx( -1 ),
    m_contextMenu(this),
    m_tabParams( defaultIS, loadingIS, fontMetrics(), -1 ),
    m_contextMenuCloseId( -1 )
{
    initContextMenu();
    initTabParams();
    setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
}

bool RoadTabBar::event( QEvent *e )
{
    if (e && e->type() == QEvent::MouseButtonPress)
    {
        QMouseEvent *mouseE = static_cast<QMouseEvent*>( e );
        if (mouseE->button() == RightButton)
        {
            showContextMenu( mouseE );
        }
    }
    return QTabBar::event( e );
}

void RoadTabBar::setCurrentTab( int i )
{
    const int oldTab = currentTab();
    QTabBar::setCurrentTab( i );
    if (oldTab != currentTab())
        emit selectedNotShow( currentTab() );
}

void RoadTabBar::setCurrentTab( QTab * tab )
{
    const int oldTab = currentTab();
    QTabBar::setCurrentTab( tab );
    if (oldTab != currentTab())
        emit selectedNotShow( currentTab() );
}

void RoadTabBar::newView( View * v )
{
    RoadTab* tab = new RoadTab( m_tabParams );
    setCurrentTab( addTab( tab ) );

    connect( v,   SIGNAL( captionChanged( const QString & ) ),
             tab, SLOT( setCaption( const QString & ) ) );
    const KAction *kAct = v->actionCollection()->action( "go_stop" );
    if (kAct)
    {
        connect( kAct, SIGNAL( enabled( bool ) ), tab, SLOT( setLoading( bool ) ) );
        connect( tab, SIGNAL( changed() ), this, SLOT( tabChanged() ) );
    }

    QString text = v->caption();
    if ( text.isEmpty() )
        text = v->part()->url().prettyURL();
    tab->setCaption( text );

    tabChanged();
}

void RoadTabBar::tabChanged()
{
    // The following is a HACK:
    // To ensure that tabs are displayed correctly after changing just about
    // anything in the tab bar, we need to call layoutTabs() and update().
    // These, however, seem to overwrite the effect of makeVisible(), which
    // means that newly created tabs make be scrolled off the tab bar.
    // We can't call makeVisible() directly because it's private.
    // Thus, we force a makeVisible() call with a dummy resizeEvent().
    layoutTabs();
    update();
    resizeEvent( 0 );
}

void RoadTabBar::removeTab( int idx )
{
    QTabBar::removeTab( tab( idx ) );
    tabChanged();
}

void RoadTabBar::initContextMenu()
{
    QAction *qa = new QAction( QString::null, i18n( "&Close Tab" ), 0, this, "contextTabClose" );
    connect( qa, SIGNAL( activated() ), this, SLOT( contextClose() ) );
    qa->addTo( &m_contextMenu );
    m_contextMenuCloseId = m_contextMenu.idAt( 0 );

    qa = new QAction( QString::null, i18n( "&New tab" ), 0, this, "contextTabNew" );
    connect( qa, SIGNAL( activated() ), this, SIGNAL( requestNewView() ) );
    qa->addTo( &m_contextMenu );
}

void RoadTabBar::initTabParams()
{
    KConfig *global = KGlobal::config();
    global->setGroup( "Tabbed Browsing" );
    m_tabParams.maxWidth = global->readNumEntry( "MaxTabWidth", 50 );
}

void RoadTabBar::showContextMenu( QMouseEvent *e )
{
    QTab *tab = selectTab( e->pos() );
    m_contextMenuTabIdx = index( tab );
    if (m_contextMenuTabIdx < 0)
        return;
    m_contextMenu.setItemEnabled( m_contextMenuCloseId, ( count() > 1 ) ); // disable the "close" option if there is only one tab open
    m_contextMenu.exec(e->globalPos());
}

void RoadTabBar::contextClose()
{
    emit closed( m_contextMenuTabIdx );
    m_contextMenuTabIdx = -1;
}

int RoadTabBar::index( QTab *tab )
{
    // HACK, but I can't find a nicer way
    if (!tab)
        return -1;
    blockSignals( true );
    int current = currentTab();
    setCurrentTab( tab );
    int retVal = currentTab();
    setCurrentTab( current );
    blockSignals( false );
    return retVal;
}

///////////////////////////////////////////////////////////////////////

RoadTab::RoadTab( const RoadTabParams &params, const QString &text )
    : QTab( params.defaultIconSet, text ),
    m_params( params )
{}

void RoadTab::setLoading( bool b )
{
    setIconSet( (b) ? m_params.loadingIconSet : m_params.defaultIconSet );
    emit changed();
}

void RoadTab::setCaption( const QString &caption )
{
    setText( formatCaption( caption ) );
    emit changed();
}

QString RoadTab::formatCaption( const QString& text )
{
    QString caption( text );
    if ( caption.isEmpty() )
        caption = i18n( "Blank" );
    // the caption may not exceed a given width, so shorten it if necessary
    // the following is inefficient, but clear, correct, and unlikely to be a bottleneck
    if ( m_params.fontMetrics.width( caption ) > m_params.maxWidth )
    {
        const QString ellipsis = "...";
        for( int newLen = caption.length() - 1;
             m_params.fontMetrics.width( caption + ellipsis ) > m_params.maxWidth;
             --newLen )
        {
            caption.truncate( newLen );
        }
        caption += ellipsis;
    }
    return caption;
}

#include "roadtabbar.moc"
