/*  This file is part of the KDE project
    Copyright (C) 2006 basyskom GmbH info@basyskom.com
    Copyright (C) 2006 ROAD GmbH info@road-gmbh.de

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifdef KONQ_GUI_ROAD

#include "urlcombo.h"

#if (QT_VERSION < 0x030000)
#include <qt3compat.h>
#endif

#include <qcombobox.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qstring.h>

#include <qfile.h>

#include <kglobal.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include <kurl.h>


class TrailingListItem : public QListBoxText
{
        public:
                TrailingListItem(const QString &text, QListBox *listbox): QListBoxText(listbox){
                        if(!text.isEmpty())
                                setText(text);
                }

                int width( const QListBox* lb ) const
                {
                        int w = lb ? lb->fontMetrics().width( text() ) + 6 : 0;
                        return QMIN( w, listBox()->width() );
                }


        protected:
                void paint(QPainter *p){
                        QFontMetrics fm = p->fontMetrics();
                        QString orig = text();
                        int trailing = fm.boundingRect("...").width();
                        int offset = 3; //copied from QListBoxText::paint
                        int maxWidth = listBox()->width() - 15;

                        uint index = 0;
                        QString truncated = "";
                        while( index < orig.length() && ( ( fm.boundingRect( truncated + orig[index] ).width() + trailing + offset ) <  maxWidth ) )
                        {
                                truncated += orig[index++];
                        }

                        if(index < orig.length() - 1){
                                truncated += "...";
                        }

                        p->drawText( offset, fm.ascent() + fm.leading()/2, truncated );

                }

};

//////////////////////////////////////////////////////////////////////////////////


URLCombo::URLCombo( QWidget* parent, const char* name )
        : QComboBox( true, parent, name )
        , m_justGotFocus( false )
{
        focusProxy()->installEventFilter( this );
        setAutoCompletion(true);
        setDuplicatesEnabled(false);

        // I need the information when the text in the internal QLineEdit was changed
        // to modify the information in the history..
        connect( this, SIGNAL( textChanged( const QString& ) ),
                 this, SLOT( slotTextChanged( const QString& ) ) );

	//QTimer::singleShot(0,this,SLOT(initHistoryList()));
        initHistoryList();
}

URLCombo::~URLCombo()
{
        kdDebug() << "~URLCombo" << endl;
        URLHistoryBuffer::self().saveURLHistory();
}


void URLCombo::initHistoryList()
{
       URLHistoryBuffer::self().loadURLHistory();
       updateHistoryList();
}

void URLCombo::clearURLHistory()
{
       URLHistoryBuffer::self().clearURLHistory();
}

void URLCombo::addURL(const QString& url)
{
      if (!url.isEmpty())
      {
          URLHistoryBuffer::self().addURL(url);
          updateHistoryList();
          setCurrentItem( 0 ); 
      }
}

void URLCombo::updateHistoryList( const QStringList& list )
{
        QStringList urlList = list;
        
        listBox()->clear();
        
        if ( urlList.isEmpty() )
                urlList = URLHistoryBuffer::self().history();

        for ( QStringList::ConstIterator it = urlList.begin(); it != urlList.end(); ++it ) {
                new TrailingListItem( *it, listBox() );  
        }

        listBox()->triggerUpdate( true );
}


void URLCombo::slotTextChanged( const QString& text )
{
        //qDebug( "TextChanged: %s", text.latin1() );

        QString enteredText = text;
        QString highlightedText = lineEdit()->markedText();
        if (highlightedText!= enteredText)
        {
            int pos = enteredText.find(highlightedText);
            enteredText.remove( pos, highlightedText.length() );
        }

        // if text is empty or starts with protocol -> show full list 
        bool containsProtocol = false;
        QString http = "http://";
        if (http.startsWith(enteredText) || enteredText.startsWith(http)) containsProtocol=true;
        http = "https://";
        if (http.startsWith(enteredText) || enteredText.startsWith(http)) containsProtocol=true;
        http = "ftp://";
        if (http.startsWith(enteredText) || enteredText.startsWith(http)) containsProtocol=true;
        http = "file:///";
        if (http.startsWith(enteredText) || enteredText.startsWith(http)) containsProtocol=true;
        
        KURL url = enteredText;
        QString domain = enteredText;
        if (!url.host().isEmpty())
              domain = url.host();
	if (domain.startsWith("www.")) domain.remove( 0, 4 );
	if (domain.startsWith("ftp.")) domain.remove( 0, 4 );
        //qDebug( "domain of changed text: %s", domain.latin1() );

        QStringList list;
        QStringList tmpList;
        // get domain only matching current text
        tmpList = URLHistoryBuffer::self().appendHistoryDomains( tmpList, enteredText );
        list = tmpList.grep( QString("www.") + domain );
        if (list.count()==0)
            list = tmpList.grep( domain );

	// 1) get all URLs which match the text entered by the user
	if (!enteredText.isEmpty())
        {
		tmpList = URLHistoryBuffer::self().history().grep(domain,false);
                QStringList::ConstIterator it;
                for( it = tmpList.begin(); it != tmpList.end(); ++it )
                     if (!list.contains(*it))
                          list.append(*it);
        }

        // 2) append history entries of all domains (without subpages)
        list = URLHistoryBuffer::self().appendHistoryDomains( list, enteredText );
        // 3) if entered text without any protocol, remove protocol from list
        if (!containsProtocol)
            list = URLHistoryBuffer::self().removeProtocolFromList( list );

        if ( !list.isEmpty() ){
                //qDebug( "Update History!! (%d items)", list.count() );
                updateHistoryList( list );
        }
        else
                listBox()->clear();
}


// *** Overloaded Members
void URLCombo::keyPressEvent( QKeyEvent* evt )
{
        switch ( evt->key() ){
                case Qt::Key_Up:
                case Qt::Key_Down:
                        popup();
                        return;
                case Qt::Key_Enter:
                case Qt::Key_Return:
                        //qDebug( "Return pressed" );
                        emit signalAccepted( currentText() );
                        URLHistoryBuffer::self().addURL( currentText() );
                        break;
                //default: 
                        //popup();
        }

        // Fall-Trough: Use parent keyhandler
        QComboBox::keyPressEvent(evt);
}

bool URLCombo::eventFilter( QObject *o, QEvent *e )
{
    // When we receive focus, select all text to make it easier to replace the
    //  existing URL.
    // This is awkward, firstly because *we* don't get focus, our focusProxy does,
    //  and secondly because, if the focus is shifted due to a mouse click, the
    //  mouse pressed/released events arrive *after* the FocusIn event (thus, the
    //  mouse click deselected what the FocusIn selected unless we block it).
    // This is a HACK and will fail if the focus isn't changed through the mouse.
    if ( o == focusProxy() )
    {
        if ( e->type() == QEvent::FocusIn )
        {
            static_cast<QLineEdit*>(focusProxy())->selectAll();
            m_justGotFocus = true;
        }
        else if ( m_justGotFocus && e->type() == QEvent::MouseButtonPress )
        {
            m_justGotFocus = false;
            return true;
        }
    }
    return QComboBox::eventFilter( o, e );
}


//////////////////////////////////////////////////////////////////////////////////


namespace{
        static const uint g_maxElementsInHistoryBuffer = 100;
}

URLHistoryBuffer URLHistoryBuffer::instance;

URLHistoryBuffer& URLHistoryBuffer::URLHistoryBuffer::self()
{
        return instance;
}


URLHistoryBuffer::URLHistoryBuffer()
        : QObject( NULL, NULL ), m_maxElementsInBuffer( g_maxElementsInHistoryBuffer )
{
}


URLHistoryBuffer::~URLHistoryBuffer()
{
   //    saveURLHistory();
}


void URLHistoryBuffer::loadURLHistory()
{
        m_bufferFilename = storageFileNameWithPath();

        QFile file( m_bufferFilename );
        if ( file.open( IO_ReadOnly ) ) {
                QDataStream stream( &file );
                stream >> m_content;
                file.close();
        }
}

void URLHistoryBuffer::saveURLHistory()
{
        kdDebug() << "URL HIstory content: " << m_content.count() << endl;
        QFile file( m_bufferFilename ); // FIXME: Set Destination for store..
        if ( file.open( IO_WriteOnly ) ) {
                QDataStream stream( &file );
                stream << m_content;
                file.close();
        }
        debugOutput();
}

void URLHistoryBuffer::clearURLHistory()
{
        m_content.clear();
}



void URLHistoryBuffer::addURL( const QString& _url )
{
        kdDebug() << "URLHistoryBuffer::addURL() " << _url << endl;
        QString url = _url;
        // This should never happen.. Just to be sure..
        if ( m_content.count() > g_maxElementsInHistoryBuffer )
                m_content.clear();

         KURL kurl = url;
         if (!kurl.isValid() || url.isEmpty())
                return;

        // Add or remove stuff which belongs to a correct URL
        if ( !url.contains( "://" ) ) // stupid test.. hopefully not too stupid!
                url = "http://" + url;
        //qDebug( "URLHistoryBuffer: Corrected URL-String: %s", url.latin1() );

        // If the same URL stored without trailing '/', remove it.
        // This one will replace it..
        if ( url.at( url.length() - 1 ) == '/' ){
                QString url_to_remove = url;
                url_to_remove.remove( url.length() - 1, 1 );
                //qDebug( "URLHistoryBuffer:: url to remove: %s", url_to_remove.latin1() );
                QStringList::Iterator it = m_content.find( url_to_remove );
                if ( it != m_content.end() )
                        m_content.remove(  m_content.find( url_to_remove ) );
        }

        // if we have url containing www. check that we don't duplicate an entry without www(krfcdate_0)
        if (kurl.host().startsWith("www.")) {
                QString url_to_remove = kurl.host() + "/" + kurl.path();
                url_to_remove.remove(0,4);
                url_to_remove = kurl.protocol() + "://" + url_to_remove;
                //qDebug( "URLHistoryBuffer:: url to remove: %s", url_to_remove.latin1() );
                QStringList::Iterator it = m_content.find( url_to_remove );
                if ( it != m_content.end() )
                        m_content.remove(  m_content.find( url_to_remove ) );
        }

        if ( url.startsWith( "file:///" ) )
        {
                QString url_to_remove = url;
                url_to_remove.remove( 5, 2 );
                //qDebug( "URLHistoryBuffer:: url to remove: %s", url_to_remove.latin1() );
                QStringList::Iterator it = m_content.find( url_to_remove );
                if ( it != m_content.end() )
                        m_content.remove( m_content.find( url_to_remove ) );
        }
                
        
        // LRU-Ageing: Move used URL on top of the list. Thus, we removing it here
        // and we will add it below..
        int pos = m_content.findIndex( url );
        if ( pos >= 0 ){
                //qDebug( "URLHistoryBuffer: found string at pos: %d", pos );
                m_content.remove ( m_content.at( pos ) );
        }

        // Handle maximum size of list
        if ( m_content.count() == g_maxElementsInHistoryBuffer )
                m_content.remove( m_content.last() );

        //qDebug( "URLHistoryBuffer: append string on top" );
        m_content.prepend( url );
}


QStringList URLHistoryBuffer::containsSubString( const QString substring ) const
{
        return m_content.grep( substring, false );
}

QStringList URLHistoryBuffer::removeProtocolFromList(const QStringList& list) const
{
        QStringList tmpList ;
        QString tmpString;
        KURL url;
        QStringList::ConstIterator it;
        for( it = list.begin(); it != list.end(); ++it )
        {
		// in KURL we can get the url without domain => convert to KURL
                url = (*it);
                tmpString = url.host() + url.path();
                if ( tmpString.startsWith("www.") )
                     tmpString.remove(0,4);

                if (!tmpList.contains(tmpString) && !tmpString.isEmpty())
                    tmpList.append(tmpString);
        }
        return tmpList;
}

QStringList URLHistoryBuffer::appendHistoryDomains(const QStringList& list, const QString& currenturl) const
{
        QStringList tmpList = list;
        QString tmpString;
        KURL url;

        // append all other domains
        QStringList::ConstIterator it;
        QString domain;
        for( it = m_content.begin(); it != m_content.end(); ++it )
        {
                url = (*it);
                if (url.protocol() != "file" && !url.protocol().isEmpty())
                {
                  domain = url.host();
                  if (domain.contains("www.")) domain.remove(0,4);
                  tmpString = url.protocol() + "://" + domain;
                  if (!tmpList.contains(tmpString) && 
                             !tmpList.contains(tmpString+"/") && 
                             !tmpList.contains(url.protocol()+"://www."+ url.host()+"/") &&
                             !tmpList.contains(url.protocol()+"://"+ url.host()+"/") )
                     tmpList.append(url.protocol() + "://" + url.host() +"/");
                }
        }
        return tmpList;
}


QStringList URLHistoryBuffer::history() const
{
        return m_content;
}

void URLHistoryBuffer::debugOutput()
{
        QStringList::Iterator it;
        uint i = 0;
        for( it = m_content.begin(); it != m_content.end(); ++it )
                printf( "+++++> History(%d): %s\n", i++, (*it).latin1() );
}

QString URLHistoryBuffer::storageFileNameWithPath()
{
        QString dir = KGlobal::dirs()->saveLocation( "data" ) + "konq-embed/urlhistory.buf";
        return dir;
}

#include "urlcombo.moc"
#endif