/*  This file is part of the KDE project
    Copyright (C) 2006 basyskom GmbH info@basyskom.com
    Copyright (C) 2006 ROAD GmbH info@road-gmbh.de

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/


#ifndef URLCOMBO_H
#define URLCOMBO_H

#include <qcombobox.h>

/**
This class provides a history URL entry line with as-you-type suggestions and a
history drop-down list. The content of the drop-down list depends on the conent of the
entry line. If the line is empty, the whole history is shown. Otherwise just URLs are shown which
have substrings related to the entry line.<br>
Use the UP/Down keys or mouse to activate the drop-down list.
This URL-Line uses the class URLHistoryBuffer internally to receive a list of URLs.

How to use:
<code>
m_pUrlLine = new URLEntryLine( this );
Q_CHECK_PTR( m_pUrlLine );

m_pUrlLine->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding,QSizePolicy::Fixed ) );
hbox_layout->addWidget( m_pUrlLine );
</code>

@author Stefan Eilers
*/
class URLCombo : public QComboBox
{
        Q_OBJECT
public:
        URLCombo( QWidget* parent=NULL, const char* name=NULL );
        virtual ~URLCombo();

public slots:
        virtual void clearURLHistory();
        virtual void addURL(const QString& url);

signals:
        /** Emitted if the URL text was accepted */ 
        void signalAccepted( const QString& text );

protected:
        void keyPressEvent(QKeyEvent* e);
        virtual bool eventFilter( QObject *o, QEvent *e );

protected slots:
        virtual void slotTextChanged( const QString& text );
        virtual void updateHistoryList( const QStringList& list = QStringList() );
        virtual void initHistoryList();
        
private:
        bool m_justGotFocus;
};



/**
Stores history information with automatic "least-recently-used" ageing mechanism and removing
of duplicates.
This class behaves like a Singleton Pattern!
@author Stefan Eilers
*/
class URLHistoryBuffer : public QObject
{
Q_OBJECT
public:
        /** Returns a reference to this singleton */
        static URLHistoryBuffer& self();

        /** Add URL into this buffer if it not already contained.. */
        virtual void addURL( const QString& url );

        /** Get all URL's containing the given substring */
        QStringList containsSubString( const QString substring ) const;
        /** Get all URL's without the protocol string */
        QStringList removeProtocolFromList(const QStringList& list) const;
        /** Get all URL's with their domain only plus all urls for domain of given url */
        QStringList appendHistoryDomains(const QStringList& list, const QString& currenturl) const;

        /** Returns the stored history list */
        QStringList history() const; 
        /** loads the url history from file */
        virtual void loadURLHistory();
        /** saves the url history to file */
        virtual void saveURLHistory();
        /** removes all history entries */
        virtual void clearURLHistory();
        
protected:
        URLHistoryBuffer();
        virtual ~URLHistoryBuffer();
        virtual void debugOutput();
        virtual QString storageFileNameWithPath();

private:
        static URLHistoryBuffer instance;

        uint m_maxElementsInBuffer;
        QStringList m_content;
        QString m_bufferFilename;
};


#endif
